﻿using System;
using Syncfusion.DocIO.DLS;

namespace Grouped
{
	class Program
	{
		private static string BaseDir { get; set; }

		static void Main()
		{
			BaseDir = AppDomain.CurrentDomain.BaseDirectory;
			var inPath = BaseDir + @"Docs\11.docx";
			var outPath = BaseDir + @"Docs\2.docx";

			WordDocument document = new WordDocument(inPath);
			foreach (WSection section in document.Sections)
			{
				WTextBody sectionBody = section.Body;
				IterateTextBody(sectionBody);
			}
			document.Save(outPath);
            System.Diagnostics.Process.Start(outPath);
		}

		private static void IterateTextBody(WTextBody textBody)
		{
			for (int i = 0; i < textBody.ChildEntities.Count; i++)
			{
				IEntity bodyItemEntity = textBody.ChildEntities[i];
				switch (bodyItemEntity.EntityType)
				{
					case EntityType.Table:
						IterateTable(bodyItemEntity as WTable);
						break;
				}
			}
		}

		private static void IterateTable(IWTable table)
		{
			var columns = 2;
			for (int i = 0; i < columns; i++)
			{
				MergeCells(table, 2, i);
			}
		}

		private static void MergeCells(IWTable table, int startRow, int startColumn)
		{
			var startText = string.Empty;		
			bool isNewGroup = true;
			int i = startRow;
            //Integer variable which is used to hold the vertical merge starting row index.
            int mergeStartRowIndex = startRow;
			for (;i < table.Rows.Count; i++)
			{
				var par = table[i, startColumn].LastParagraph;
				var currentText = par.Text;

				if (isNewGroup && currentText.Contains("_$_"))
				{
					startText = par.Text;
                    mergeStartRowIndex = i;
                    //Changed the cell index as "startColumn" instead of 0.
                    //This is the reason why the second column merging not done properly.
                    table.Rows[i].Cells[startColumn].CellFormat.VerticalMerge = CellMerge.Start;
					isNewGroup = false;
					continue;
				}

				if (!string.IsNullOrEmpty(startText) && currentText.Equals(startText))
				{
					table.Rows[i].Cells[startColumn].CellFormat.VerticalMerge = CellMerge.Continue;
				}
				else
				{
					isNewGroup = true;
                    --i;
                    //Get the cell from vertical merge start row index instead of getting its previous row.
                    //For example if we merge three row cells then we should get the cell from first row instead middle row.
                    var cell = table[mergeStartRowIndex, startColumn];
					ReplaceAndAlignText(cell);
				}
			}

            ReplaceAndAlignText(table[mergeStartRowIndex, startColumn]);

		}

		private static void ReplaceAndAlignText(WTableCell cell)
		{
			var par = cell.LastParagraph;
			var text = par.Text.Split(new [] {"_$_"}, StringSplitOptions.None)[1];
			par.Text = text;
			cell.CellFormat.VerticalAlignment = VerticalAlignment.Middle;
		}
	}
}
